#!/usr/bin/python
#Copyright 2017-2018 James Healey
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
def BookmarkToggle(*args):
    desktop = XSCRIPTCONTEXT.getDesktop()
    model = desktop.getCurrentComponent()
    doc = XSCRIPTCONTEXT.getDocument()
    try:
        text = model.Text
    except:
        # The macro has been called externally but LibreOffice was not running at the time
        return None
    # If bookmarks already exist as annotations assume Hide is required
    # otherwise assume Show
    oEnum = model.getTextFields().createEnumeration()
    oEnum_count = 0
    while oEnum.hasMoreElements():
        oField = oEnum.nextElement()
        if oField.supportsService('com.sun.star.text.TextField.Annotation'):
            if oField.Author == "@BookmarkToggle":
                doc.Text.removeTextContent(oField)
                oEnum_count += 1
    if oEnum_count != 0:
        return None
    bookmarks = model.getBookmarks()
    for bookmark in bookmarks:
        bm_anchor = bookmark.getAnchor()
        anchorText = bm_anchor.Text
        cursor = anchorText.createTextCursorByRange(bm_anchor.Start)
        #LibreOffice deletes the bookmark along with the Annotation if they are in the same position
        #Normally moving to the right 1 character sorts this out
        #If we can't move to the right, insert a space so that we can
        if cursor.goRight(1,False): #Something to the right continue
            pass
        else:                       #Nothing to the right, insert a space
            anchorText.insertString(cursor, " ", False)
        anno = model.createInstance("com.sun.star.text.textfield.Annotation")
        anno.Content = bookmark.Name
        anno.Author = "@BookmarkToggle"
        anchorText.insertTextContent(cursor,anno,False)
    return None